/*
	SourceMod Anti-Cheat SpinHack Module
	Copyright (C) 2011 GoD-Tony
	Copyright (C) 2007-2011 CodingDirect LLC

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
	Credit to EKS for his algorithm.
	Possible feature: If a player's sensitivity is very high then ignore them.
*/

#define SPIN_DETECTIONS		15		// Seconds of non-stop spinning before spinhack is detected
#define SPIN_ANGLE_CHANGE	2250000	// (1500^2) Max angle deviation over  one second before being flagged

//- Global Variables -//

new Handle:g_hCVarSpinHack = INVALID_HANDLE;
new bool:g_bSpinHackEnabled;
new g_iSpinHackMode;

new Handle:g_hSpinLoop = INVALID_HANDLE;

new Float:g_fLastAngles[MAXPLAYERS+1][3];
new Float:g_fAngleDiff[MAXPLAYERS+1];
new g_iSpinCount[MAXPLAYERS+1];

//- Plugin Functions -//

SpinHack_OnPluginStart()
{
	g_hCVarSpinHack = CreateConVar("smac_spinhack", "0", "SpinHack detection module. (0 = Disabled, 1 = Warn Admins, 2 = Ban)");
	
	SpinHack_CvarChange(g_hCVarSpinHack, "", "");
	HookConVarChange(g_hCVarSpinHack, SpinHack_CvarChange);
}

SpinHack_OnClientDisconnect(client)
{
	g_iSpinCount[client] = 0;
}

public SpinHack_CvarChange(Handle:convar, const String:oldValue[], const String:newValue[])
{
	g_iSpinHackMode = GetConVarInt(convar);

	if (g_iSpinHackMode && !g_bSpinHackEnabled)
		SpinHack_Enable();
	else if (!g_iSpinHackMode && g_bSpinHackEnabled)
		SpinHack_Disable();
}

public Action:SpinHack_CheckSpins(Handle:timer)
{
	for (new i = 1; i <= MaxClients; i++)
	{
		if (g_bInGame[i] && IsPlayerAlive(i))
		{
			if ((g_fAngleDiff[i] * g_fTickRate) < SPIN_ANGLE_CHANGE)
				g_iSpinCount[i] = 0;
			else if (g_iSpinCount[i]++ >= SPIN_DETECTIONS)
				SpinHack_Detected(i);
			
			g_fAngleDiff[i] = 0.0;
		}
	}
	
	return Plugin_Continue;
}

SpinHack_OnPlayerRunCmd(client, buttons, Float:angles[3])
{
	if (!g_bSpinHackEnabled || buttons & IN_LEFT || buttons & IN_RIGHT)
		return;
	
	g_fAngleDiff[client] += GetVectorDistance(g_fLastAngles[client], angles, true);
	
	g_fLastAngles[client][0] = angles[0];
	g_fLastAngles[client][1] = angles[1];
	g_fLastAngles[client][2] = angles[2];
}

SpinHack_Detected(client)
{
	// Prevent a flood of detections.
	if (g_iSpinCount[client] > SPIN_DETECTIONS + 1)
		return;
	
	new String:f_sName[64], String:f_sAuthID[64], String:f_sIP[64];
	GetClientName(client, f_sName, sizeof(f_sName));
	GetClientAuthString(client, f_sAuthID, sizeof(f_sAuthID));
	GetClientIP(client, f_sIP, sizeof(f_sIP));
	
	switch (g_iSpinHackMode)
	{
		case 1: // Warn Admins.
		{
			SMAC_Log("%N (ID: %s | IP: %s) is suspected of using a spinhack.", client, f_sAuthID, f_sIP);
			SMAC_PrintToChatAdmins("%t", SMAC_SPINHACKDETECTED, f_sName);
		}
		case 2: // Ban.
		{
			SMAC_Log("%N (ID: %s | IP: %s) was banned for using a spinhack.", client, f_sAuthID, f_sIP);
			SMAC_Ban(client, 0, SMAC_BANNED, "SMAC: SpinHack Detected");
		}
	}
}

SpinHack_Enable()
{
	g_bSpinHackEnabled = true;
	
	if (g_hSpinLoop == INVALID_HANDLE)
		g_hSpinLoop = CreateTimer(1.0, SpinHack_CheckSpins, _, TIMER_REPEAT);
}

SpinHack_Disable()
{
	g_bSpinHackEnabled = false;
	
	if (g_hSpinLoop != INVALID_HANDLE)
	{
		KillTimer(g_hSpinLoop);
		g_hSpinLoop = INVALID_HANDLE;
	}
}

//- EoF -//
